@@ -12,6 +12,8 @@ extern "C" {
 }
 #endif
 
+#include <stdlib.h>
+
 #ifndef PERL_VERSION
 #    include <patchlevel.h>
 #    if !(defined(PERL_VERSION) || (PERL_SUBVERSION > 0 && defined(PATCHLEVEL)))
@@ -244,12 +246,22 @@ srl_build_encoder_struct(pTHX_ HV *opt)
         if ( svp && SvTRUE(*svp) )
             enc->flags |= SRL_F_COMPRESS_SNAPPY;
 
+        svp = hv_fetchs(opt, "snappy_incr", 0);
+        if ( svp && SvTRUE(*svp) )
+            enc->flags |= SRL_F_COMPRESS_SNAPPY_INCREMENTAL;
+
         svp = hv_fetchs(opt, "undef_unknown", 0);
         if ( svp && SvTRUE(*svp) ) {
             undef_unknown = 1;
             enc->flags |= SRL_F_UNDEF_UNKNOWN;
         }
 
+        svp = hv_fetchs(opt, "sort_keys", 0);
+        if ( svp && SvTRUE(*svp) ) {
+            undef_unknown = 1;
+            enc->flags |= SRL_F_SORT_KEYS;
+        }
+
         svp = hv_fetchs(opt, "stringify_unknown", 0);
         if ( svp && SvTRUE(*svp) ) {
             if (expect_false( undef_unknown )) {
@@ -326,6 +338,8 @@ srl_write_header(pTHX_ srl_encoder_t *enc)
                                  | (
                                     SRL_ENC_HAVE_OPTION(enc, SRL_F_COMPRESS_SNAPPY)
                                     ? SRL_PROTOCOL_ENCODING_SNAPPY
+                                    : SRL_ENC_HAVE_OPTION(enc, SRL_F_COMPRESS_SNAPPY_INCREMENTAL)
+                                    ? SRL_PROTOCOL_ENCODING_SNAPPY_INCREMENTAL
                                     : SRL_PROTOCOL_ENCODING_RAW
                                  );
 
@@ -458,7 +472,7 @@ srl_dump_data_structure(pTHX_ srl_encoder_t *enc, SV *src)
     /* Register our structure for destruction on scope exit */
     SAVEDESTRUCTOR_X(&srl_destructor_hook, (void *)enc);
 
-    if (!SRL_ENC_HAVE_OPTION(enc, SRL_F_COMPRESS_SNAPPY)) {
+    if (!SRL_ENC_HAVE_OPTION(enc, (SRL_F_COMPRESS_SNAPPY | SRL_F_COMPRESS_SNAPPY_INCREMENTAL))) {
         srl_write_header(aTHX_ enc);
         srl_dump_sv(aTHX_ enc, src);
         srl_fixup_weakrefs(aTHX_ enc);
@@ -488,11 +502,17 @@ srl_dump_data_structure(pTHX_ srl_encoder_t *enc, SV *src)
         }
         else {
             char *old_buf;
+            char *varint_start= NULL;
+            char *varint_end;
             uint32_t dest_len;
 
             /* Get uncompressed payload and total packet output (after compression) lengths */
             dest_len = csnappy_max_compressed_length(uncompressed_body_length) + sereal_header_len + 1;
 
+            if ( SRL_ENC_HAVE_OPTION(enc, SRL_F_COMPRESS_SNAPPY_INCREMENTAL ) ) {
+                dest_len += SRL_MAX_VARINT_LENGTH;
+            }
+
             /* Lazy working buffer alloc */
             if (expect_false( enc->snappy_workmem == NULL )) {
                 /* Cleaned up automatically by the cleanup handler */
@@ -515,12 +535,43 @@ srl_dump_data_structure(pTHX_ srl_encoder_t *enc, SV *src)
             Copy(old_buf, enc->pos, sereal_header_len, char);
             enc->pos += sereal_header_len;
 
+            if ( SRL_ENC_HAVE_OPTION(enc, SRL_F_COMPRESS_SNAPPY_INCREMENTAL ) ) {
+                varint_start= enc->pos;
+                srl_buf_cat_varint_nocheck(aTHX_ enc, 0, dest_len);
+                varint_end= enc->pos - 1;
+            }
+
             /*
              * fprintf(stderr, "'%u' %u %u\n", enc->pos - enc->buf_start, uncompressed_body_length, (uncompressed_body_length+sereal_header_len));
              * fprintf(stdout, "%7s!%1s\n", old_buf, old_buf+6);
              */
             csnappy_compress(old_buf+sereal_header_len, (uint32_t)uncompressed_body_length, enc->pos, &dest_len,
                              enc->snappy_workmem, CSNAPPY_WORKMEM_BYTES_POWER_OF_TWO);
+
+            if ( varint_start ) {
+                /* overwrite the max size varint with the real size of the compressed data */
+                UV n= dest_len;
+                while (n >= 0x80) {                      /* while we are larger than 7 bits long */
+                    *varint_start++ = (n & 0x7f) | 0x80; /* write out the least significant 7 bits, set the high bit */
+                    n = n >> 7;                          /* shift off the 7 least significant bits */
+                }
+                /* if it is the same size we can use a canonical varint */
+                if ( varint_start == varint_end ) {
+                    *varint_start = n;                     /* encode the last 7 bits without the high bit being set */
+                } else {
+                    /* if not we produce a non-canonical varint, basically we stuff
+                     * 0 bits (via 0x80) into the "tail" of the varint, until we can
+                     * stick in a null to terminate the sequence. This means that the
+                     * varint is effectively "self-padding", and we only need special
+                     * logic in the encoder - a decoder will happily process a non-canonical
+                     * varint with no problem */
+                    *varint_start++ = (n & 0x7f) | 0x80;
+                    while ( varint_start < varint_end )
+                        *varint_start++ = 0x80;
+                    *varint_start= 0;
+                }
+            }
+
             /* fprintf(stderr, "%u, %u %u %u\n", dest_len, enc->pos[0], enc->pos[1], enc->pos[2]); */
             assert(dest_len != 0);
             Safefree(old_buf);
@@ -689,6 +740,36 @@ srl_dump_av(pTHX_ srl_encoder_t *enc, AV *src, U32 refcount)
     }
 }
 
+/* compare hash entries, used when all keys are bytestrings */
+static int
+he_cmp_fast(const void *a_, const void *b_)
+{
+    /* even though we are called as a callback from qsort there is
+     * no need for a dTHX here, we don't use anything that needs it */
+    int cmp;
+
+    HE *a = *(HE **)a_;
+    HE *b = *(HE **)b_;
+
+    STRLEN la = HeKLEN (a);
+    STRLEN lb = HeKLEN (b);
+
+    if (!(cmp = memcmp (HeKEY (b), HeKEY (a), lb < la ? lb : la)))
+        cmp = lb - la;
+
+    return cmp;
+}
+
+/* compare hash entries, used when some keys are sv's or utf8 */
+static int
+he_cmp_slow(const void *a, const void *b)
+{
+    /* we are called as a callback from qsort, so no pTHX
+     * is possible in our argument signature, so we need to do a
+     * dTHX; here ourselves. */
+    dTHX;
+    return sv_cmp( HeSVKEY_force( *(HE **)b), HeSVKEY_force( *(HE **)a ) );
+}
 
 static SRL_INLINE void
 srl_dump_hv(pTHX_ srl_encoder_t *enc, HV *src, U32 refcount)
@@ -697,7 +778,7 @@ srl_dump_hv(pTHX_ srl_encoder_t *enc, HV *src, U32 refcount)
     const int do_share_keys = HvSHAREKEYS((SV *)src);
     UV n;
 
-    if ( SvMAGICAL(src) ) {
+    if ( SvMAGICAL(src) || SRL_ENC_HAVE_OPTION(enc, SRL_F_SORT_KEYS) ) {
         UV i;
         /* for tied hashes, we have to iterate to find the number of entries. Alas... */
         (void)hv_iterinit(src); /* return value not reliable according to API docs */
@@ -717,18 +798,58 @@ srl_dump_hv(pTHX_ srl_encoder_t *enc, HV *src, U32 refcount)
 
         (void)hv_iterinit(src); /* return value not reliable according to API docs */
         i = 0;
-        while ((he = hv_iternext(src))) {
-            SV *v;
-            if (expect_false( i == n ))
+        if (SRL_ENC_HAVE_OPTION(enc, SRL_F_SORT_KEYS)) {
+            HE **he_array;
+            int fast = 1;
+            Newxz(he_array, n, HE*);
+            SAVEFREEPV(he_array);
+            while ((he = hv_iternext(src))) {
+                if (expect_false( i == n ))
+                    croak("Panic: Trying to dump a tied hash that has a different number of keys in each iteration is just daft. Do not do that.");
+                he_array[i++]= he;
+                if (HeKLEN (he) < 0 || HeKUTF8 (he))
+                    fast = 0;
+            }
+            if (expect_false( i != n ))
                 croak("Panic: Trying to dump a tied hash that has a different number of keys in each iteration is just daft. Do not do that.");
-            v= hv_iterval(src, he);
-            srl_dump_hk(aTHX_ enc, he, do_share_keys);
-            CALL_SRL_DUMP_SV(enc, v);
-            ++i;
-        }
-        if (expect_false( i != n ))
-            croak("Panic: Trying to dump a tied hash that has a different number of keys in each iteration is just daft. Do not do that.");
+            if (fast) {
+                qsort(he_array, n, sizeof (HE *), he_cmp_fast);
+            } else {
+                /* hack to forcefully disable "use bytes" */
+                COP cop= *PL_curcop;
+                cop.op_private= 0;
 
+                ENTER;
+                SAVETMPS;
+
+                SAVEVPTR (PL_curcop);
+                PL_curcop= &cop;
+
+                qsort(he_array, n, sizeof (HE *), he_cmp_slow);
+
+                FREETMPS;
+                LEAVE;
+            }
+            for ( i= 0; i < n ; i++ ) {
+                SV *v;
+                he= he_array[i];
+                v= hv_iterval(src, he);
+                srl_dump_hk(aTHX_ enc, he, do_share_keys);
+                CALL_SRL_DUMP_SV(enc, v);
+            }
+        } else {
+            while ((he = hv_iternext(src))) {
+                SV *v;
+                if (expect_false( i == n ))
+                    croak("Panic: Trying to dump a tied hash that has a different number of keys in each iteration is just daft. Do not do that.");
+                v= hv_iterval(src, he);
+                srl_dump_hk(aTHX_ enc, he, do_share_keys);
+                CALL_SRL_DUMP_SV(enc, v);
+                ++i;
+            }
+            if (expect_false( i != n ))
+                croak("Panic: Trying to dump a tied hash that has a different number of keys in each iteration is just daft. Do not do that.");
+        }
     } else {
         n= HvUSEDKEYS(src);
         /* heuristic: n = ~min size of n values;
